header "Dynamic Pushdown Networks"
theory DPN 
imports Main "common/LTS"
begin
text_raw {*\label{thy:DPN}*}

declare predicate2I[HOL.rule del, Pure.rule del]

subsection "Model Definition"
text {*
  A {\em Dynamic Pushdown Network} (DPN) \cite{BMT05} is a system of pushdown rules 
  over states from @{typ "'Q"} and stack symbols from @{typ "'\<Gamma>"}, where each 
  pushdown rule may spawn additional processes. Rules are labeled by elements of type 
  @{typ "'L"}
  *}

datatype ('P,'\<Gamma>,'L) pushdown_rule =
  NOSPAWN 'P '\<Gamma> 'L 'P "'\<Gamma> list" ("_,_ \<hookrightarrow>\<^bsub>_\<^esub> _,_" 51) |
  SPAWN 'P '\<Gamma> 'L 'P "'\<Gamma> list" 'P "'\<Gamma> list" ("_,_ \<hookrightarrow>\<^bsub>_\<^esub> _,_ \<sharp> _,_" 51)

notation NOSPAWN ("_,_ \<hookrightarrow>\<^sub>_ _,_" 51)
notation SPAWN ("_,_ \<hookrightarrow>\<^sub>_ _,_ \<sharp> _,_" 51)

types ('Q,'\<Gamma>,'L) dpn = "('Q,'\<Gamma>,'L) pushdown_rule set"

text {* We fix the finiteness assumption of the set of rules in a locale.
  Note that we do not assume the base types of states, stack symbols, or labels to be 
  finite.
  However, the finiteness assumption of the set of rules implies that the sets 
  of {\em used} control states, stack symbols, and labels are finite.
*}
locale DPN =
  fixes \<Delta> :: "('Q,'\<Gamma>,'L) dpn"
  assumes ruleset_finite[simp, intro!]: "finite \<Delta>"

end
