header "DPNs with Initial Configuration"
theory DPN_c0
imports WellNested
begin
text_raw {*\label{thy:DPN_c0}*}

subsection {* DPNs with Initial Configuration *}
text {*
  In the following locale, we fix a DPN with an initial configuration, and a 
  list of lock-stacks. We assume that the initial configuration is well-nested w.r.t.
  the list of lock-stacks. 
  
  This is the model we are able to analyze with our acquisition history based 
  techniques, that assume well-nestedness.

  Note that we -- up to now -- do not show that there exists a non-trivial instance
  of this locale. Such a proof would support the trust in that the model we formalize 
  here is really the intended model.
*}
locale LDPN_c0 = LDPN +
  constrains \<Delta> :: "('P,'\<Gamma>,'L,'X::finite) ldpn"
  fixes c0 :: "('P,'\<Gamma>) conf"           -- "Initial configuration"
  fixes \<mu>0 :: "'X list list"           -- "Locks held at the start configuration"
  assumes wellnested: "wn_c \<Delta> c0 \<mu>0"   -- "Start configuration must be well-nested"
begin
  subsubsection "Reachable Configurations"
  definition "reachable == { c . \<exists>w. (c0,w,c)\<in>dpntrc \<Delta> }"
  definition "reachablels == { (c,X) . \<exists>w. ((c0,locks_\<mu> \<mu>0),w,(c,X))\<in>ldpntrc \<Delta> }"

  lemma reachablels_subset: "(c,X)\<in>reachablels \<Longrightarrow> c\<in>reachable" 
    by (auto simp add: reachablels_def reachable_def intro: ldpntrc_subset)

  lemma reachable_wn: 
    "\<lbrakk>(c,X)\<in>reachablels; !!\<mu>. \<lbrakk>wn_c \<Delta> c \<mu>; X=locks_\<mu> \<mu>\<rbrakk> \<Longrightarrow> P\<rbrakk> \<Longrightarrow> P"
    apply (unfold reachablels_def)
    apply simp
    apply (erule exE)
    apply (erule wnc_preserve)
    apply (rule wellnested)
    apply blast
    done

  lemma reachablels_triv[simp]: "(c0, locks_\<mu> \<mu>0)\<in>reachablels"
    by (unfold reachablels_def) (auto intro: exI[of _ "[]"])

end

end
